clc
clear all
close all
% Note: The following line should be changed to the directory where "exp_data.mat" file is located in the PC drive
load(fullfile('','Panasonic DataSheet','exp_data.mat'))
batt_id = 'Lithium Ion High Energy Density 3.3Ah';
label = 'LiIon';
current_label = {'0p66A','1p65A','3p3A','6p6A','3p3A','3p3A','3p3A','3p3A'};
current = [0.66 1.65 3.3 6.6 3.3 3.3 3.3 3.3];
temperature_label = {'25C','25C','25C','25C','N20C','N10C','0C','40C'};
temperature = [25 25 25 25 -20 -10 0 40];
line_colors = {'r','g','b','c','m','y','b','r'};
markerType1 = {'none', 'none', 'o', 'none', 'none', 'none', 'none', 'none'};
figure(1);
grid on ;
subplot(2,1,1);hold on;
for j=1:4
data = ['LiIon_' current_label{j} '_' temperature_label{j} '_Panasonic'];
plot(exp_data.(data)(:,1),exp_data.(data)(:,2),'color',line_colors{j},'Marker',markerType1{j});
end
title(['Discharge Characteristics of LiIon Battery' sprintf('\n\n') 'Discharge Characteristics at Constant 25 \circC']);
xlabel('Discharge Capacity [mAh]')
ylabel('Voltage [V]');
legend({'0.2C','0.5C','1.0C','2.0C'},'Location','EastOutside')
grid on
hold off;

subplot(2,1,2);hold on;
for j=[3 5:8]
data = ['LiIon_' current_label{j} '_' temperature_label{j} '_Panasonic'];
plot(exp_data.(data)(:,1),exp_data.(data)(:,2),'color',line_colors{j},'Marker',markerType1{j});
end
title('Temperature Characteristics at Constant 3.3A');
xlabel('Discharge Capacity [mAh]')
ylabel('Voltage [V]');
ylim([2.5 4.5]);
legend({'25 \circC','-20 \circC','-10 \circC','0 \circC','40 \circC'},'Location','EastOutside')
grid on
hold off;
%% Step 2: Normalize State-of-Charge (SOC) Data
ref_exp = 3;
val_exp = logical([1 0 0 0 1 0 0 0]);
ref_curr = current == current(ref_exp);
ref_temp = temperature == temperature(ref_exp);

[sort_current, sort_index_current] = sort(current(ref_temp));
[sort_temp, sort_index_temp] = sort(temperature(ref_curr));
N = length(current); % Number of experiments
for i=1:N
    x.(['curr' current_label{i} '_temp' temperature_label{i}]) = ...
            exp_data.([label '_' current_label{i} '_' temperature_label{i} '_Panasonic'])(:,1)/...
            exp_data.([label '_' current_label{i} '_' temperature_label{i} '_Panasonic'])(end,1);
    % Calculate actual capacity for each datasheet
    correct_cap.(['curr' current_label{i} '_temp' temperature_label{i}]) = ...
            exp_data.([label '_' current_label{i} '_' temperature_label{i} '_Panasonic'])(end,1);
end
%% Normalized SOC Data
figure(2);
count = 1;
hold on
for i=find(ref_temp)
    p1 = plot(x.(['curr' current_label{i} '_temp' temperature_label{i}]), exp_data.([label '_' current_label{i} '_' temperature_label{ref_exp} '_Panasonic'])(:,2));
    leg{count}=['I = ' num2str(current(i)) ' A'];
    count = count+1;
end
legend(leg,'Location','EastOutside');
xlabel('1-SOC'); xlim([0 1]); ylabel('Voltage [V]');ylim([2.5 4.2]); title(['Normalized SOC Data' sprintf('\n\n') batt_id ' ' temperature_label{ref_exp}]);
%% Step 3: Fit Curves
SOC_LUT = (0:.01:1)';
SOCbkpts = 0:.2:1;
for i=find(ref_temp)
    fitObj.(['fit' current_label{i}]) = ...
        fit(x.(['curr' current_label{i} '_temp' temperature_label{i}]),...
        exp_data.([label '_' current_label{i} '_' temperature_label{ref_exp} '_Panasonic'])(:,2),'smoothingspline');
end
for i=find(ref_curr)
    fitObj.(['fit' temperature_label{i}]) = ...
        fit(x.(['curr' current_label{i} '_temp' temperature_label{i}]),...
        exp_data.([label '_' current_label{ref_exp} '_' temperature_label{i} '_Panasonic'])(:,2),'smoothingspline');
end
Em_MAT = [];
for i=find(ref_temp)
    Em_MAT = [Em_MAT fitObj.(['fit' current_label{i}])(SOC_LUT)];
end
figure(3)
hold on
for i=1:10:length(SOC_LUT)
    plot(sort_current,Em_MAT(i,sort_index_current))%,colorV{i},'Marker','o','MarkerFaceColor',colorF{i})
end
xlabel('Current [A]')
ylabel('Voltage [V]')
legend(num2str(flipud(SOC_LUT(1:10:end))),'Location','eastoutside')
title(['Curve Fitting' sprintf('\n\n') batt_id sprintf('\n') ...
    'Em Matrix of Cell/Module Voltage vs. Discharge ' ...
    sprintf('\n') 'Current at Different SOC']);
%% Step 4: Extrapolate Open Circuit Voltage
R0_refTemp = [];
for i=1:length(SOC_LUT)
    % Fit a line to V=f(I)
    fitSOC.(['SOC' num2str(i)]) = fit(sort_current',Em_MAT(i,sort_index_current)','poly1');
end
Em = [];
for i=1:length(SOC_LUT)
    % Em = f(0)
    Em = [Em fitSOC.(['SOC' num2str(i)])(0)];
end
Em = Em';
%% Step 5: Determine Battery Voltage and Resistance at Different Temperatures
%Plot discharge curves at different temperatures 
figure(4)
hold on
for i=find(ref_curr)
    plot(x.(['curr' current_label{i} '_temp' temperature_label{i}]), ...
        exp_data.([label '_' current_label{ref_exp} '_' temperature_label{i} '_Panasonic'])(:,2));
end
legend(temperature_label(find(ref_curr)),'Location','eastoutside');
xlabel('1-SOC'); xlim([0 1]); ylabel('Voltage [V]'); title([batt_id ' ' current_label{ref_exp}]);
title(['Figure 4 - Battery Voltage' sprintf('\n\n') 'Battery Voltage at Different Temperatures Under Constant Load']);
%%
R0_LUT = [];
for i=find(ref_curr)
    % Create fit object for V vs. SOC
    voltVsSOC.(['temp' temperature_label{i}]) = fitObj.(['fit' temperature_label{i}])(SOC_LUT);
    % Calculate R0(SOC,T) assuming linear behavior R0 = DeltaV / I
    R0.(['temp' temperature_label{i}]) = (Em - voltVsSOC.(['temp' temperature_label{i}]))./current(ref_exp);
    % Construct LUT
    R0_LUT = [R0_LUT R0.(['temp' temperature_label{i}])];
end
if ~isempty(find(SOC_LUT==0.9, 1))
    R0_LUT(SOC_LUT>0.9,:) = repmat(R0_LUT(SOC_LUT == 0.9,:),length(R0_LUT(SOC_LUT>0.9,:)),1);
else
    [closestTo0p9, locClosestTo0p9] = min(abs(SOC_LUT-0.9));
    R0_LUT(SOC_LUT>closestTo0p9,:) = repmat(R0_LUT(locClosestTo0p9,:),...
                                     length(R0_LUT(SOC_LUT>closestTo0p9,:)),1);
end
R0_LUT = max(R0_LUT,0);
T_LUT = 273.15 + temperature(ref_curr);
[T_LUT1,idx] = sort(T_LUT);
xtmp=R0_LUT';
R0_LUT1(1:length(T_LUT),:) = xtmp(idx,:);
% Plot battery resistance at different temperatures

figure(5)
grid on ;
plot(SOC_LUT,R0_LUT)
legend({'25 \circC','-20 \circC','-10 \circC','0 \circC','40 \circC'},'Location','eastoutside');
xlabel('1-SOC'); xlim([0 1]); ylabel('Resistance [\Omega]');
title(['Battery Resistance' sprintf('\n\n') 'Battery Resistance vs. SOC at Different Temperatures']);
%% Step 7: Fit Battery Resistance

R0_brkpoints=[];
for q=1:length(SOCbkpts)
    R0_brkpoints=[R0_brkpoints find(SOC_LUT==SOCbkpts(q))];
end
R0_LUT_bkpts = [];
counter = 1;
for i=find(ref_curr)
    R0_LUT_bkpts = [R0_LUT_bkpts;R0_LUT1(counter,R0_brkpoints)];
    counter = counter+1;
end
%%
%% Lookup Table Breakpoints

Battery.SOC_LUT = [SOC_LUT]';
Battery.Temperature_LUT = T_LUT1;

%% Em Branch Properties (OCV, Capacity)

% Battery capacity
Battery.Capacity_LUT = [2.2 2.8 2.96 3.3 3.3]; %Ampere*hours

% Em open-circuit voltage vs SOC rows and T columns
Battery.Em_LUT = [Em Em Em Em Em]; %Volts
Battery.Em_LUT = flipud(Battery.Em_LUT);

%% Terminal Resistance Properties

% R0 resistance vs SOC rows and T columns
Battery.R0_LUT = R0_LUT1'; %Ohms

   
%% Thermal Properties

% Cell dimensions and sizes
cell_thickness = 18.5*10^-3; %m
cell_width = 18.5*10^-3; %m
cell_height = 65.3*10^-3; %m

% Cell surface area
cell_area = (2*pi()*(cell_width/2)^2) + (pi()*cell_width*cell_height); %m^2

% Cell volume
cell_volume = pi()* (cell_width/2)^2 * cell_height; %m^3

% Cell mass
Battery.cell_mass = 0.0485; %kg

% Volumetric heat capacity
% assumes uniform heat capacity throughout the cell
% ref: J. Electrochemical Society 158 (8) A955-A969 (2011) pA962
cell_rho_Cp = 2.04E6; %J/m3/K

% Specific Heat
% Battery.cell_Cp_heat = (cell_rho_Cp * cell_volume)/Battery.cell_mass; %J/kg/K
Battery.cell_Cp_heat =830; %J/Kg/K
% Convective heat transfer coefficient
% For natural convection this number should be in the range of 5 to 25
h_conv = 10; %W/m^2/K
h_conv_end = 10; %W/m^2/K

%% Initial Conditions

% Heating Power
P_PTC = -1000; % Watt
% Charge deficit
Battery.Qe_init = 0; %Ampere*hours

% Initial Temperatures
Battery.T_init = 40 + 273.15; %K
T_ambient= 40 + 273.15; %K
T_coolant_initial= 40 + 273.15; %K

% Cells in series
Ns=28;
% Cells in parallel
Np=42;
% Mass of Coolant
m_f= 11; %Kg
    %m_f= 14.78904742; % in case of additional cooling plate from above
% Specific heat of Coolant
cp_f= 3403; %J/Kg/K
% Thermal Resistance between battery and coolant.
R_bc=0.019; %K/W
    %R_bc=0.009253239; % in case of additional cooling plate from above
% Global Heat Transfer Coefficient of the Radiator
Kr_Ar=153.6; % W/K
% Max Battery Voltage
V_b_max=(Em(1)*Ns)+4;
% Battery Discharge Current in Amperes
I_d= 1*138.6;% Amperes
I_charging= -63; % Amperes


%% Panda BEVintage
% VPE_s1 input data

%% Vehicle Body Total Load (Computed)
m_e = 1.0359e+03;
c_0 = 145.0911;
c_1 = 0;
c_2 = 0.4764;
g = 9.81;

%% Mapped Motor
% Electrical Torque - Nominal & Maximal
w_n = 2895 /60*2*pi;
T_m_n = 40;
eta_n = 0.83 *100; %@ w_n & T_m_n
P_m_n = w_n * T_m_n
T_m_max = 66;
P_m_max = w_n*T_m_max*1.1
tau_m = 0.02;
% Electrical Losses - Single Efficiency
Pl_iron = 0; %IPM
Pl_eln = 100;
Pl_copper_n = (1-eta_n/100)*P_m_n -Pl_eln -Pl_iron
k_T = Pl_copper_n /T_m_n^2
% Mechanical
J_m = .005e-3;
beta_m = 1e-5;

%% Transmission
r_e = 0.27;
T_r = 8.67;

%% Longitudinal Driver
V_max = 120 /3.6;
K_p = 15;
K_i = 1;
K_aw = 0.1;
K_ff = 0.05;
K_g = 0.01;
tau_e = 0.03;

%% Battery
V_DC = 96;
%% Driving Cycles Modified
ECE_modified.xdata= [0	11	13	21	27	48	54	78	89	110	113	123	135	143	158	168	175	186	188	196	198.5	219.5	225.5	249.5	255	276	279	289	301	305	320	325	332	343	345	353	355.5	376.5	382.5	406.5	412	433	436	446	458	462	477	482	489	500	502	510	512.5	533.5	539.5	563.5	569	590	593	603	615	619	634	639	646]';
ECE_modified.ydata= [0	0	4.167	4.167	0	0	8.889	8.889	0	0	4.167	13.89	13.89	9.722	9.722	0	0	0	4.167	4.167	0	0	8.889	8.889	0	0	4.167	13.89	13.89	9.722	9.722	0	0	0	4.167	4.167	0	0	8.889	8.889	0	0	4.167	13.89	13.89	9.722	9.722	0	0	0	4.167	4.167	0	0	8.889	8.889	0	0	4.167	13.89	13.89	9.722	9.722	0	0]';

NEDC_modified.xdata=[0	11	13	21	27	48	54	78	89	110	113	123	135	143	158	168	175	186	188	196	198.5	219.5	225.5	249.5	255	276	279	289	301	305	320	325	332	343	345	353	355.5	376.5	382.5	406.5	412	433	436	446	458	462	477	482	489	500	502	510	512.5	533.5	539.5	563.5	569	590	593	603	615	619	634	639	646	666	696.5	746.5	754.5	823.5	830	880	897.5	927.5	937.5	947.5	963.5	971.5	981.5	1000.5]'; 
NEDC_modified.ydata=[0	0	4.167	4.167	0	0	8.889	8.889	0	0	4.167	13.89	13.89	9.722	9.722	0	0	0	4.167	4.167	0	0	8.889	8.889	0	0	4.167	13.89	13.89	9.722	9.722	0	0	0	4.167	4.167	0	0	8.889	8.889	0	0	4.167	13.89	13.89	9.722	9.722	0	0	0	4.167	4.167	0	0	8.889	8.889	0	0	4.167	13.89	13.89	9.722	9.722	0	0	0	19.44	19.44	13.89	13.89	19.44	19.44	27.78	27.78	33.33	33.33	22.22	13.89	0	0]';

